export interface StorageCursor {
    limit(number_: number): StorageCursor
    sort(parameters: Record<string, any>): StorageCursor
    skip(offset: number): StorageCursor
    toArray(): Promise<any[]>
}

export class ErrStorage extends Error {}
export class ErrNoSession extends ErrStorage {}

export abstract class Storage {
    protected _dsn: string

    constructor(dsn: string) {
        this._dsn = dsn
    }

    abstract find(name: string, query: Record<string, unknown>): Promise<StorageCursor>

    abstract save(name: string, uniqKey: string, data: Record<string, unknown>): Promise<string>

    abstract createSession(): StorageSession

    abstract count(name: string, query?: Record<string, unknown>): Promise<number>

    abstract remove(collectionName: string, uniqKeyName: string, uniqKey: string): Promise<boolean>
}

export abstract class StorageSession {
    abstract start(options?: Record<string, unknown>): Promise<void>
    abstract commit(fn: () => any, options?: Record<string, unknown>): Promise<void>
}
